#!/usr/bin/env python

###########################################################################
#   Copyright (C) 2008-2009 by John Dickson
#   <captsens@gmail.com>
#
#   Licensed under the GNU Public Licence.  See LICENCE file that comes with this distribution
###########################################################################

"""
Command-line script to produce a list of all files to be backed up from a JCR installation.
The script must be provided with the path to the JCR config file.

Only projects at or beyond review status are backed up, and for each of these, the following files/
directories will be added to the list:

- Project data directory (include java list, original file list and diff list)
- All files under project diff directory, so long as they are selected for review and have a status of
  'changed'
- All files in the original file set for the project, so long as they are selected for review and have
  a status of 'removed'
- All files in the modified file set for the project, so long as they are selected for review and have
  a status of 'added'
"""

import sys, os

from paste.deploy import appconfig
from pylons import config
from jcr.config.environment import load_environment

from jcr import model

# ================================================================================================
class StdOutWriter(object):
    def writeLn(self, text):
        print text
    def writeStatusLn(self, text):
        print >> sys.stderr, line

# ------------------------------------------------------------------------------------------------
def usage(msg=None):
    if msg:
        print msg
    print """
Produces a list of all files to be backed up from a JCR installation.

Usage:
    mkBackupList.py <configFile>
e.g.
    mkBackupList.py path/to/jcr.ini"""
    sys.exit(1)

# ------------------------------------------------------------------------------------------------
def _getEligibleProjects():
    p = model.project.Project
    return model.project.findByStatus([p.PROJECT_STATUS_REVIEW_PREPARATION, p.PROJECT_STATUS_REVIEW_MEETING,
            p.PROJECT_STATUS_POST_REVIEW, p.PROJECT_STATUS_COMPLETE])

# ------------------------------------------------------------------------------------------------
def addBackupFiles(project, filesToBackup, writer):
    writer.writeStatusLn('Processing %s' % project)

    def addFile(*pathElements):
        filesToBackup[os.path.join(*pathElements)] = True

    projectVcs = model.vcs.getVcs(project)
    for file in projectVcs.getBackupFilePaths(project):
        addFile(file)
    
    #projectDirs = model.vcs.tarvcs.ProjectDirectories(project)
    
    #for projectFile in model.projectfile.findForProject(project.systemId):
        #if projectFile.changeType == projectFile.CHANGE_TYPE_CHANGED:
            #addFile(projectDirs.diffDirectoryRelative, projectFile.path)
        #elif projectFile.changeType == projectFile.CHANGE_TYPE_ADDED:
            ## it's in the modified fileset
            #addFile(projectDirs.modifiedDirectoryRelative, projectFile.path)
        #else:
            ## removed - it's in the original fileset
            #addFile(projectDirs.originalDirectoryRelative, projectFile.path)

# ------------------------------------------------------------------------------------------------
def main(args, writer):
    if len(args) != 1:
        usage()

    filename = os.path.abspath(args[0])
    if not os.path.isfile(filename):
        usage('Config file not found: %s' % filename)
    conf = appconfig('config:' + filename)
    load_environment(conf.global_conf, conf.local_conf)
    model.orm._buildSchema()
    model.vcs.init(config)

    try:
        # Contains True keyed on file path (relative to JCR base directory).  Using dict to suppress dupes
        filesToBackup = {}

        for project in model.project.findProjectsRequiringBackup():
            addBackupFiles(project, filesToBackup, writer)

        for file in sorted(filesToBackup.keys()):
            writer.writeLn(file)

    except KeyboardInterrupt:
        pass

# ------------------------------------------------------------------------------------------------
if __name__ == '__main__':
    main(sys.argv[1:], StdOutWriter())