/****************************************************************************
 * Database create script
 *
 * Database:         jcr
 * DBMS:             postgres8
 *****************************************************************************/

/*===========================================================================
 * Sequence:         tableids
 * Description:      Sequence used to generate system IDs (primary keys) for all
 *                   tables
 *===========================================================================*/
CREATE SEQUENCE tableids;


/*===========================================================================
 * Table:            systemuser
 * Description:      Records details of a user authorised to use JCR
 *===========================================================================*/
CREATE TABLE systemuser (
    systemid             INTEGER NOT NULL,
    username             VARCHAR(20) NOT NULL,
    name                 VARCHAR(50) NOT NULL,
    issuperuser          CHAR(1) NOT NULL,
    password             VARCHAR(32) NOT NULL,
    email                VARCHAR(100),
    enabled              CHAR(1) NOT NULL,
    CONSTRAINT pk_systemuser PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            repository
 * Description:      Represents a repository in a source control system
 *                   (currently supports Subversion only)
 *===========================================================================*/
CREATE TABLE repository (
    systemid             INTEGER NOT NULL,
    name                 VARCHAR(100) NOT NULL,
    url                  VARCHAR(500) NOT NULL,
    vcstype              VARCHAR(3) NOT NULL,
    username             VARCHAR(50),
    password             VARCHAR(200),
    enabled              CHAR(1) NOT NULL,
    CONSTRAINT pk_repository PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            fileset
 * Description:      Represents a set of files that can be used by one or more
 *                   review projects.
 *                   Filesets always use the Tarball VCS
 *===========================================================================*/
CREATE TABLE fileset (
    systemid             INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    directory            VARCHAR(500),
    baseline             VARCHAR(500),
    archivestatus        CHAR(1),
    CONSTRAINT pk_fileset PRIMARY KEY (systemid),
    CONSTRAINT fk_fileset_systemuser FOREIGN KEY (owner) REFERENCES systemuser
);


/*===========================================================================
 * Table:            project
 * Description:      Represents a code review project
 *===========================================================================*/
CREATE TABLE project (
    systemid             INTEGER NOT NULL,
    name                 VARCHAR(100) NOT NULL,
    owner                INTEGER NOT NULL,
    ownerdelegate        INTEGER,
    createdate           TIMESTAMP NOT NULL,
    directory            VARCHAR(200) NOT NULL,
    currentstatus        VARCHAR(10) NOT NULL,
    originalfileset      INTEGER,
    modifiedfileset      INTEGER,
    ignorewhitespace     CHAR(1),
    notes                TEXT,
    repository           INTEGER,
    pathinrepository     VARCHAR(500),
    originalRevisionType VARCHAR(2),
    originalRevisionParam VARCHAR(20),
    modifiedRevisionType VARCHAR(2),
    modifiedRevisionParam VARCHAR(20),
    archivestatus        CHAR(1),
    CONSTRAINT pk_project PRIMARY KEY (systemid),
    CONSTRAINT fk_project_systemuser FOREIGN KEY (owner) REFERENCES systemuser,
    CONSTRAINT fk_project_systemuser_delegate FOREIGN KEY (ownerdelegate) REFERENCES systemuser,
    CONSTRAINT fk_project_originalfileset FOREIGN KEY (originalfileset) REFERENCES fileset,
    CONSTRAINT fk_project_modifiedfileset FOREIGN KEY (modifiedfileset) REFERENCES fileset,
    CONSTRAINT fk_project_repository FOREIGN KEY (repository) REFERENCES repository
);


/*===========================================================================
 * Table:            projectstatus
 * Description:      Records the history of status changes to each project
 *===========================================================================*/
CREATE TABLE projectstatus (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    changedby            INTEGER NOT NULL,
    changedate           TIMESTAMP NOT NULL,
    newstatus            VARCHAR(2) NOT NULL,
    CONSTRAINT pk_projectstatus PRIMARY KEY (systemid),
    CONSTRAINT fk_projectstatus_project FOREIGN KEY (project) REFERENCES project,
    CONSTRAINT fk_projectstatus_changedBy FOREIGN KEY (changedby) REFERENCES systemuser
);


/*===========================================================================
 * Table:            projectthread
 * Description:      Groups together project-level comments with a common title,
 *                   like an email thread
 *===========================================================================*/
CREATE TABLE projectthread (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    threadtitle          VARCHAR(100) NOT NULL,
    CONSTRAINT pk_projectthread PRIMARY KEY (systemid),
    CONSTRAINT fk_projectthread_project FOREIGN KEY (project) REFERENCES project,
    CONSTRAINT fk_projectthread_systemuser FOREIGN KEY (owner) REFERENCES systemuser
);

CREATE INDEX in_projectthread_project ON projectthread (
    project
);

CREATE INDEX in_projectthread_owner ON projectthread (
    owner
);


/*===========================================================================
 * Table:            projectcomment
 * Description:      Review comments made against a project thread
 *===========================================================================*/
CREATE TABLE projectcomment (
    systemid             INTEGER NOT NULL,
    projectthread        INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    comment              TEXT NOT NULL,
    reviewdate           TIMESTAMP,
    reviewcomment        TEXT,
    reviewer             INTEGER,
    reviewdecision       VARCHAR(2),
    reviewactionowner    INTEGER,
    actiondate           TIMESTAMP,
    actioncomment        TEXT,
    actioner             INTEGER,
    actionscomplete      CHAR(1),
    CONSTRAINT pk_projectcomment PRIMARY KEY (systemid),
    CONSTRAINT fk_projectcomment_projectthread FOREIGN KEY (projectthread) REFERENCES projectthread,
    CONSTRAINT fk_projectcomment_systemuser_owner FOREIGN KEY (owner) REFERENCES systemuser,
    CONSTRAINT fk_projectcom_systemuser_reviewer FOREIGN KEY (reviewer) REFERENCES systemuser,
    CONSTRAINT fk_projectcom_systemuser_revaction FOREIGN KEY (reviewactionowner) REFERENCES systemuser,
    CONSTRAINT fk_projectcom_systemuser_actioner FOREIGN KEY (actioner) REFERENCES systemuser
);

CREATE INDEX in_projectcomment_projectthread ON projectcomment (
    projectthread
);

CREATE INDEX in_projectcomment_owner ON projectcomment (
    owner
);

CREATE INDEX in_projectcomment_reviewer ON projectcomment (
    reviewer
);

CREATE INDEX in_projectcomment_revaction ON projectcomment (
    reviewactionowner
);

CREATE INDEX in_projectcomment_actioner ON projectcomment (
    actioner
);


/*===========================================================================
 * Table:            projectfile
 * Description:      Represents a file within a project that has been selected
 *                   for review
 *===========================================================================*/
CREATE TABLE projectfile (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    path                 VARCHAR(500) NOT NULL,
    originalpath         VARCHAR(500),
    changetype           CHAR(1) NOT NULL,
    notes                TEXT,
    reviewpriority       SMALLINT,
    numlines             INTEGER,
    changepercent        SMALLINT,
    numchunks            INTEGER,
    CONSTRAINT pk_projectfile PRIMARY KEY (systemid),
    CONSTRAINT fk_projectfile_project FOREIGN KEY (project) REFERENCES project
);

CREATE UNIQUE INDEX in_projectfile_project_path ON projectfile (
    project,
    path
);


/*===========================================================================
 * Table:            filecomment
 * Description:      Review comments made against lines in a file
 *===========================================================================*/
CREATE TABLE filecomment (
    systemid             INTEGER NOT NULL,
    projectfile          INTEGER NOT NULL,
    linenumber           INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    comment              TEXT NOT NULL,
    reviewdate           TIMESTAMP,
    reviewcomment        TEXT,
    reviewer             INTEGER,
    reviewdecision       VARCHAR(2),
    reviewactionowner    INTEGER,
    actiondate           TIMESTAMP,
    actioncomment        TEXT,
    actioner             INTEGER,
    actionscomplete      CHAR(1),
    CONSTRAINT pk_filecomment PRIMARY KEY (systemid),
    CONSTRAINT fk_filecomment_projectfile FOREIGN KEY (projectfile) REFERENCES projectfile,
    CONSTRAINT fk_filecomment_systemuser_owner FOREIGN KEY (owner) REFERENCES systemuser,
    CONSTRAINT fk_filecom_systemuser_reviewer FOREIGN KEY (reviewer) REFERENCES systemuser,
    CONSTRAINT fk_filecom_systemuser_revaction FOREIGN KEY (reviewactionowner) REFERENCES systemuser,
    CONSTRAINT fk_filecom_systemuser_actioner FOREIGN KEY (actioner) REFERENCES systemuser
);

CREATE INDEX in_filecomment_owner ON filecomment (
    owner
);

CREATE INDEX in_filecomment_reviewer ON filecomment (
    reviewer
);

CREATE INDEX in_filecomment_revaction ON filecomment (
    reviewactionowner
);

CREATE INDEX in_filecomment_actioner ON filecomment (
    actioner
);


/*===========================================================================
 * Table:            fileaction
 * Description:      Tracks actions on project files by particular users, so we
 *                   can show users which files they have yet to review
 *===========================================================================*/
CREATE TABLE fileaction (
    systemid             INTEGER NOT NULL,
    projectfile          INTEGER NOT NULL,
    systemuser           INTEGER NOT NULL,
    views                SMALLINT NOT NULL,
    commentviews         SMALLINT NOT NULL,
    comments             SMALLINT NOT NULL,
    score                SMALLINT NOT NULL,
    CONSTRAINT pk_fileaction PRIMARY KEY (systemid),
    CONSTRAINT fk_fileaction_projectfile FOREIGN KEY (projectfile) REFERENCES projectfile,
    CONSTRAINT fk_fileaction_systemuser FOREIGN KEY (systemuser) REFERENCES systemuser
);

CREATE INDEX in_fileaction_file_user ON fileaction (
    projectfile,
    systemuser
);


/*===========================================================================
 * Table:            projectreviewer
 * Description:      Records system users allocated to review projects
 *===========================================================================*/
CREATE TABLE projectreviewer (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    systemuser           INTEGER NOT NULL,
    reviewrole           VARCHAR(2),
    notifydate           TIMESTAMP,
    reviewcompletedate   TIMESTAMP,
    CONSTRAINT pk_projectreviewer PRIMARY KEY (systemid),
    CONSTRAINT un_projectreviewer_project_reviewer UNIQUE (project, systemuser),
    CONSTRAINT fk_projectreviewer_project FOREIGN KEY (project) REFERENCES project,
    CONSTRAINT fk_projectreviewer_systemuser FOREIGN KEY (systemuser) REFERENCES systemuser
);

CREATE INDEX in_projectreviewer_systemuser ON projectreviewer (
    systemuser
);

