/****************************************************************************
 * Database create script
 *
 * Database:         jcr
 * DBMS:             sqlite
 *****************************************************************************/

/*===========================================================================
 * Table:            systemuser
 * Description:      Records details of a user authorised to use JCR
 *===========================================================================*/
CREATE TABLE systemuser (
    systemid             INTEGER NOT NULL,
    username             VARCHAR(20) NOT NULL,
    name                 VARCHAR(50) NOT NULL,
    issuperuser          CHAR(1) NOT NULL,
    password             VARCHAR(32) NOT NULL,
    email                VARCHAR(100),
    enabled              CHAR(1) NOT NULL,
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            repository
 * Description:      Represents a repository in a source control system
 *                   (currently supports Subversion only)
 *===========================================================================*/
CREATE TABLE repository (
    systemid             INTEGER NOT NULL,
    name                 VARCHAR(100) NOT NULL,
    url                  VARCHAR(500) NOT NULL,
    vcstype              VARCHAR(3) NOT NULL,
    username             VARCHAR(50),
    password             VARCHAR(200),
    enabled              CHAR(1) NOT NULL,
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            fileset
 * Description:      Represents a set of files that can be used by one or more
 *                   review projects.
 *                   Filesets always use the Tarball VCS
 *===========================================================================*/
CREATE TABLE fileset (
    systemid             INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    directory            VARCHAR(500),
    baseline             VARCHAR(500),
    archivestatus        CHAR(1),
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            project
 * Description:      Represents a code review project
 *===========================================================================*/
CREATE TABLE project (
    systemid             INTEGER NOT NULL,
    name                 VARCHAR(100) NOT NULL,
    owner                INTEGER NOT NULL,
    ownerdelegate        INTEGER,
    createdate           TIMESTAMP NOT NULL,
    directory            VARCHAR(200) NOT NULL,
    currentstatus        VARCHAR(10) NOT NULL,
    originalfileset      INTEGER,
    modifiedfileset      INTEGER,
    ignorewhitespace     CHAR(1),
    notes                TEXT,
    repository           INTEGER,
    pathinrepository     VARCHAR(500),
    originalRevisionType VARCHAR(2),
    originalRevisionParam VARCHAR(20),
    modifiedRevisionType VARCHAR(2),
    modifiedRevisionParam VARCHAR(20),
    archivestatus        CHAR(1),
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            projectstatus
 * Description:      Records the history of status changes to each project
 *===========================================================================*/
CREATE TABLE projectstatus (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    changedby            INTEGER NOT NULL,
    changedate           TIMESTAMP NOT NULL,
    newstatus            VARCHAR(2) NOT NULL,
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            projectthread
 * Description:      Groups together project-level comments with a common title,
 *                   like an email thread
 *===========================================================================*/
CREATE TABLE projectthread (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    threadtitle          VARCHAR(100) NOT NULL,
    PRIMARY KEY (systemid)
);

CREATE INDEX in_projectthread_project ON projectthread (
    project
);

CREATE INDEX in_projectthread_owner ON projectthread (
    owner
);


/*===========================================================================
 * Table:            projectcomment
 * Description:      Review comments made against a project thread
 *===========================================================================*/
CREATE TABLE projectcomment (
    systemid             INTEGER NOT NULL,
    projectthread        INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    comment              TEXT NOT NULL,
    reviewdate           TIMESTAMP,
    reviewcomment        TEXT,
    reviewer             INTEGER,
    reviewdecision       VARCHAR(2),
    reviewactionowner    INTEGER,
    actiondate           TIMESTAMP,
    actioncomment        TEXT,
    actioner             INTEGER,
    actionscomplete      CHAR(1),
    PRIMARY KEY (systemid)
);

CREATE INDEX in_projectcomment_projectthread ON projectcomment (
    projectthread
);

CREATE INDEX in_projectcomment_owner ON projectcomment (
    owner
);

CREATE INDEX in_projectcomment_reviewer ON projectcomment (
    reviewer
);

CREATE INDEX in_projectcomment_revaction ON projectcomment (
    reviewactionowner
);

CREATE INDEX in_projectcomment_actioner ON projectcomment (
    actioner
);


/*===========================================================================
 * Table:            projectfile
 * Description:      Represents a file within a project that has been selected
 *                   for review
 *===========================================================================*/
CREATE TABLE projectfile (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    path                 VARCHAR(500) NOT NULL,
    originalpath         VARCHAR(500),
    changetype           CHAR(1) NOT NULL,
    notes                TEXT,
    reviewpriority       SMALLINT,
    numlines             INTEGER,
    changepercent        SMALLINT,
    numchunks            INTEGER,
    PRIMARY KEY (systemid)
);


/*===========================================================================
 * Table:            filecomment
 * Description:      Review comments made against lines in a file
 *===========================================================================*/
CREATE TABLE filecomment (
    systemid             INTEGER NOT NULL,
    projectfile          INTEGER NOT NULL,
    linenumber           INTEGER NOT NULL,
    owner                INTEGER NOT NULL,
    createdate           TIMESTAMP NOT NULL,
    comment              TEXT NOT NULL,
    reviewdate           TIMESTAMP,
    reviewcomment        TEXT,
    reviewer             INTEGER,
    reviewdecision       VARCHAR(2),
    reviewactionowner    INTEGER,
    actiondate           TIMESTAMP,
    actioncomment        TEXT,
    actioner             INTEGER,
    actionscomplete      CHAR(1),
    PRIMARY KEY (systemid)
);

CREATE INDEX in_filecomment_owner ON filecomment (
    owner
);

CREATE INDEX in_filecomment_reviewer ON filecomment (
    reviewer
);

CREATE INDEX in_filecomment_revaction ON filecomment (
    reviewactionowner
);

CREATE INDEX in_filecomment_actioner ON filecomment (
    actioner
);


/*===========================================================================
 * Table:            fileaction
 * Description:      Tracks actions on project files by particular users, so we
 *                   can show users which files they have yet to review
 *===========================================================================*/
CREATE TABLE fileaction (
    systemid             INTEGER NOT NULL,
    projectfile          INTEGER NOT NULL,
    systemuser           INTEGER NOT NULL,
    views                SMALLINT NOT NULL,
    commentviews         SMALLINT NOT NULL,
    comments             SMALLINT NOT NULL,
    score                SMALLINT NOT NULL,
    PRIMARY KEY (systemid)
);

CREATE INDEX in_fileaction_file_user ON fileaction (
    projectfile,
    systemuser
);


/*===========================================================================
 * Table:            projectreviewer
 * Description:      Records system users allocated to review projects
 *===========================================================================*/
CREATE TABLE projectreviewer (
    systemid             INTEGER NOT NULL,
    project              INTEGER NOT NULL,
    systemuser           INTEGER NOT NULL,
    reviewrole           VARCHAR(2),
    notifydate           TIMESTAMP,
    reviewcompletedate   TIMESTAMP,
    PRIMARY KEY (systemid),
    CONSTRAINT un_projectreviewer_project_reviewer UNIQUE (project, systemuser)
);

CREATE INDEX in_projectreviewer_systemuser ON projectreviewer (
    systemuser
);

