#!/usr/bin/env python

"""
(C) Copyright 2007, by John Dickson

Project Info:  http://clichart.sourceforge.net/

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
USA.

$Id: aggregate.py 581 2007-06-10 05:58:06Z johnd $


A simple script to aggregate one or more columns in a tabular (CSV or whitespace-separated)
data file or stream.

See usage()
"""

import sys, getopt, os, re
from statslib import *

OUTPUT_MIN = 'min'
OUTPUT_MAX = 'max'
OUTPUT_AVERAGE = 'av'
OUTPUT_COUNT = 'cnt'
OUTPUT_TOTAL = 'tot'
OUTPUT_FIRST = 'first'
OUTPUT_LAST = 'last'

SIGNIFICANT_DIGITS = 6

ALL_OUTPUT_TYPES = (OUTPUT_MIN, OUTPUT_MAX, OUTPUT_AVERAGE, OUTPUT_COUNT, OUTPUT_TOTAL,
        OUTPUT_FIRST, OUTPUT_LAST)

COLUMN_EXPRESSION_RE = re.compile(r'(\d+):(%s)' % '|'.join(ALL_OUTPUT_TYPES))

# ===============================================================
class ColumnStats:
    def __init__(self, columnNumber):
        self.columnNumber = columnNumber
        self.count = 0
        self.total = 0.0
        self.min = self.max = None
        self.first = self.last = None

    def accumulate(self, stringValue, lineNumber):
        """Note that value is a string"""
        try:
            value = int(stringValue)
        except:
            try:
                value = float(stringValue)
            except:
                raise InvalidDataException('Invalid number in column %d, line %d: %s' % (self.columnNumber,
                        lineNumber, stringValue))

        self.total += value
        self.count += 1
        if self.min is None or value < self.min:
            self.min = value
        if self.max is None or value > self.max:
            self.max = value
        if self.first is None:
            self.first = value
        self.last = value

    def getValue(self, outputType):
        if outputType == OUTPUT_MIN:
            return self.min or 0
        elif outputType == OUTPUT_MAX:
            return self.max or 0
        elif outputType == OUTPUT_AVERAGE:
            if self.count == 0:
                return 0.0
            return self.total / float(self.count)
        elif outputType == OUTPUT_COUNT:
            return self.count
        elif outputType == OUTPUT_TOTAL:
            return self.total
        elif outputType == OUTPUT_FIRST:
            return self.first
        elif outputType == OUTPUT_LAST:
            return self.last

# ===============================================================
def usage(msg = None):
    if msg:
        print msg
        print
    print """
A simple script to extract aggregate data (minimum, average, maximum, total,
count, first or last value) from files containing tabular data (CSV or
whitespace-separated). The output of the script will be 1 row of data for
each input file (or only one row if stdin is used).  Output is in the same
format as the input, i.e. whitespace or comma-separated.

Usage: aggregate.py [options] [inputFile*]

Options:
 -c           Data is CSV (default is whitespace-separated)
 -f           First row of file is a header row, and should be skipped
 -h           Show help (this information) and exit
 -l <columns> A comma-separated ordered list of columns and aggregate types
              to include in the output (required).  The column index is the
              0-based number of the column in the file, while the aggregate
              type is 'min', 'av', 'max', 'tot', 'cnt', 'first' or 'last'.
              Note that the count is just the number of lines in the file,
              not counting any header row.
              For example, 1:max,2:av,2:max,0:cnt,4:tot
              Simple expressions can also be used, e.g. '1:max / 3:cnt / 60'
 -p           A prefix column value to write the aggregate columns for each
              line.  Pass 1 or more of these, e.g. to provide a date column
              value.  Example: -p 12/03/2007
 -s           If no rows of data were found, silently output nothing
 --nojit      Disable Psyco Just-In-Time compiler
"""
    sys.exit(1)

# ---------------------------------------------------------------
def splitLine(line, isCsv):
    if isCsv:
        return line.split(',')
    else:
        return line.split()

# ---------------------------------------------------------------
def getColumnNumbers(columnOutput):
    """Given all the column output expressions, return the indexes of columns we need to aggregate"""
    columnNumbers = []
    for expression in columnOutput:
        for columnNumber, outputType in COLUMN_EXPRESSION_RE.findall(expression):
            columnNumber = int(columnNumber)
            if columnNumber not in columnNumbers:
                columnNumbers.append(columnNumber)
    return columnNumbers

# ---------------------------------------------------------------
def parseFile(inFile, columnOutput, isCsv, skipFirst):
    # contains ColumnStats keyed on column number
    allColumnStats = {}
    for columnNumber in getColumnNumbers(columnOutput):
        if columnNumber not in allColumnStats:
            allColumnStats[columnNumber] = ColumnStats(columnNumber)

    isFirstLine = True
    lineNumber = 0
    for line in inFile:
        lineNumber += 1
        #print line
        if isFirstLine and skipFirst:
            isFirstLine = False
            continue
        cpts = splitLine(line, isCsv)
        #print cpts
        for columnNumber, columnStats in allColumnStats.items():
            try:
                columnStats.accumulate(cpts[columnNumber], lineNumber)
            except IndexError:
                raise InvalidDataException('Cannot find column %d in line %d' % (columnNumber,
                        lineNumber))

    return allColumnStats

# ---------------------------------------------------------------
def evaluateExpression(expression, allColumnStats):
    """Given an expression containing column references like 3:tot, 0:av etc., evaluate and return based
    on the stats dict"""
    def insertColumnStatsValue(match):
        columnNumber = int(match.group(1))
        outputType = match.group(2)
        return str(allColumnStats[columnNumber].getValue(outputType))
    updatedExpression = COLUMN_EXPRESSION_RE.sub(insertColumnStatsValue, expression)
    #print updatedExpression
    try:
        return eval(updatedExpression)
    except Exception, e:
        #raise e
        raise InvalidOptionException('Invalid expression: %s' % expression)

# ---------------------------------------------------------------
def formatOutputValue(value):
    """Try to output with no more than SIGNIFICANT_DIGITS"""
    quotient, remainder = divmod(value, 1.0)
    if remainder == 0 or quotient > 10 ** SIGNIFICANT_DIGITS:
        return '%d' % quotient
    # need to work out picture string to provide SIGNIFICANT_DIGITS overall
    quotientDigits = len(str(quotient))
    if quotient == 0:
        quotientDigits = 0
    pictureString = '%' + '%d.%d' % (SIGNIFICANT_DIGITS + 1, SIGNIFICANT_DIGITS - quotientDigits) + 'f'
    return (pictureString % value).strip()

# ---------------------------------------------------------------
def printColumnStats(outFile, columnOutput, allColumnStats, isCsv, prefixColumnValues):
    outputValues = []
    outputValues.extend(prefixColumnValues)
    for expression in columnOutput:
        statsValue = evaluateExpression(expression, allColumnStats)
        outputValues.append(formatOutputValue(statsValue))
    if isCsv:
        outFile.write(', '.join(outputValues))
        outFile.write('\n')
    else:
        outFile.write(''.join(['%10s' % value for value in outputValues]))
        outFile.write('\n')

# ---------------------------------------------------------------
def processFile(inFile, outFile, columnOutput, isCsv, skipFirst, silent, prefixColumnValues):
    # contains ColumnStats keyed on column number
    allColumnStats = parseFile(inFile, columnOutput, isCsv, skipFirst)
    if allColumnStats.values()[0].count == 0:
        if not silent:
            raise InvalidDataException('No data found!')
        else:
            return
    printColumnStats(outFile, columnOutput, allColumnStats, isCsv, prefixColumnValues)

# ---------------------------------------------------------------
if __name__ == '__main__':
    opts, args = getopt.getopt(sys.argv[1:], 'cfhl:p:s', ['nojit'])
    skipFirst = False
    isCsv = False
    # contains column expressions, e.g. 3:tot, 1:cnt
    columnOutput = []
    prefixColumnValues = []
    silent = False
    enableJit = True
    for opt, arg in opts:
        if opt == '-h':
            usage()
        elif opt == '-c':
            isCsv = True
        elif opt == '-f':
            skipFirst = True
        elif opt == '-l':
            columnOutput = arg.split(',')
        elif opt == '-p':
            prefixColumnValues.append(arg)
        elif opt == '-s':
            silent = True
        elif opt == '--nojit':
            enableJit = False
    if not columnOutput:
        usage('Output column specification is required')

    enablePsyco(enableJit)
    try:
        if len(args) == 0:
            processFile(sys.stdin, sys.stdout, columnOutput, isCsv, skipFirst, silent, prefixColumnValues)
        else:
            for arg in args:
                if not os.path.exists(arg):
                    usage('Cannot locate input file: %s' % arg)
                inFile = open(arg)
                processFile(inFile, sys.stdout, columnOutput, isCsv, skipFirst, silent, prefixColumnValues)
                inFile.close()
    except InvalidOptionException, e:
        usage(e.args[0])
    except InvalidDataException, e:
        print e.args[0]