"""
(C) Copyright 2007, by John Dickson

Project Info:  http://clichart.sourceforge.net/

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
USA.

$Id: cliserverlib.py 573 2007-06-03 06:50:31Z johnd $


A library to drive clichart in its CLI server mode.  This allows you to have a single
instance of clichart started (by this library), and have all your charts generated by
that one instance - much more efficient than starting a new instance for each chart.

To use this library, construct a ClichartDriver object, call generateChart() on it as
many times as you need, then call close().  See CLICHART_OPTIONS for all the options
that can be passed to generateChart().

Expects to find one of the following in the same directory as this library:
    - clichart shell script
    - a symlink to the clichart shell script
    - clichart.bat batch file.
"""

import os, threading, Queue, time

DEBUG = False

# Allowable chart types (determines what the X axis is based on)
CHART_TYPE_DATETIME = 'd'
CHART_TYPE_VALUE = 'v'
CHART_TYPE_NONE = 'n'


# metadata for option commands to clichart.  Consists of {paramName: (commandName, methodName)}
CLICHART_OPTIONS = {
    'inputPath': ('inputPath', '_setOption'),
    'outputPath': ('outputpath', '_setOption'),
    'columnList': ('columnlist', '_setListOption'),
    'columnList2': ('columnlist2', '_setListOption'),
    'isCsv': ('csv', '_setBooleanOption'),
    'hasHeader': ('hasheader', '_setBooleanOption'),
    # NB: accepts one of the CHART_TYPE_* values above
    'chartType': (None, '_setChartTypeOption'),
    'title': ('title', '_setOption'),
    'xTitle': ('xtitle', '_setOption'),
    'yTitle': ('ytitle', '_setOption'),
    'yTitle2': ('ytitle2', '_setOption'),
    'ignoreMissingColumns': ('ignoremissing', '_setBooleanOption'),
    'ignoreDuplicateValues': ('ignoredup', '_setBooleanOption'),
    'dateFormat': ('dateformat', '_setOption'),
    'showDatapoints': ('datapoints', '_setBooleanOption'),
    'showDatapoints2': ('datapoints2', '_setBooleanOption'),
    'lineWeight': ('lineweight', '_setOption'),
    'lineWeight2': ('lineweight2', '_setOption'),
    'maxY': ('maxy', '_setOption'),
    'maxY2': ('maxy2', '_setOption'),
    'height': ('height', '_setOption'),
    'width': ('width', '_setOption'),
    'isBar': ('bar', '_setBooleanOption'),
    'isBar2': ('bar2', '_setBooleanOption'),
    'debugEcho': ('debug-echo', '_setBooleanOption'),
    'seriesTitles': ('seriestitles', '_setListOption'),
    'seriesTitles2': ('seriestitles2', '_setListOption'),
}

# ==============================================================================
class ClichartError(Exception):
    """Thrown to indicate an error returned from clichart"""
    pass

# ==============================================================================
class StdoutReader(threading.Thread):
    """A threaded reader for reading stdout and/or stderr streams (from clichart), so that we don't
    block forever waiting for stuff that never arrives.  If a queue is provided, this makes lines
    available via the queue, otherwise it just outputs the data received."""
    def __init__(self, stdout, queue):
        threading.Thread.__init__(self)
        self.stdout = stdout
        self.queue = queue
        self.setDaemon(True)
    def run(self):
        """Override the run method in Thread"""
        while True:
            line = self.stdout.readline()
            #print '  line read', line.strip(), 'EOL'
            if not line:
                # EOF, so terminate thread
                #print 'Thread terminating'
                return
            line = line.strip()
            if self.queue:
                self.queue.put(line)
            else:
                print 'stderr:', line

# ==============================================================================
class ClichartDriver:
    """The driver for interacting with clichart"""
    def __init__(self, responseTimeout = 10):
        """Start the clichart instance in CLI server mode, ready for interaction.  The responseTimeout
        is the number of seconds we'll wait for a response from clichart before throwing an exception"""
        self.responseTimeout = responseTimeout
        modulePath = __file__
        if modulePath.endswith('.pyc'):
            modulePath = modulePath[:-1]
        if os.path.islink(modulePath):
            modulePath = os.path.realpath(modulePath)
        clichartPath = os.path.join(os.path.dirname(modulePath), 'clichart')
        if clichartPath == 'clichart' and os.path.exists(clichartPath):
            # if in current directory, Linux/unix needs path
            clichartPath = './clichart'
        # want to allow finding via PATH
        if not os.path.exists(clichartPath) and not os.path.exists(clichartPath + '.bat'):
            clichartPath = 'clichart'
        commandLine = '%s --cliserver' % clichartPath
        if DEBUG:
            print commandLine
        self.stdin, self.stdout, self.stderr = os.popen3(commandLine)
        self.queue = Queue.Queue()
        # stdout goes to queue, just print stderr
        StdoutReader(self.stdout, self.queue).start()
        StdoutReader(self.stderr, None).start()
        self.__readResponse()

    def generateChart(self, clearFirst = True, **kw):
        """Call clichart to generate the chart using the parameters provided (see docstring for this class
        for details of all params).  If clearFirst is set to False, these parameters will merely override
        the options set in previous calls, otherwise the clichart defaults will be used"""
        if clearFirst:
            self.__sendCommand('clear')
        self.__processOptions(kw)
        self.__sendCommand('go')

    def close(self):
        """Shut down clichart"""
        self.__sendCommand('quit', expectResponse = False)
        self.stdin.close()
        self.stdout.close()

    def __processOptions(self, optionParams):
        for key, value in optionParams.items():
            if key in CLICHART_OPTIONS:
                command, methodName = CLICHART_OPTIONS[key]
                exec('self.%s(command, value)' % methodName)
            else:
                raise ClichartError('Invalid argument: %s' % key)

    def _setOption(self, command, value):
        self.__sendCommand(command, value)

    def _setListOption(self, command, value):
        """Value is a list of ints, and must be turned into CSV"""
        self.__sendCommand(command, ','.join(['%s' % item for item in value]))

    def _setBooleanOption(self, command, value):
        """If value is True, send the command with no value. If false, do nothing"""
        if value:
            self.__sendCommand(command)

    def _setChartTypeOption(self, command, value):
        if value == CHART_TYPE_DATETIME:
            # TODO: Can't do this yet!
            pass
        elif value == CHART_TYPE_VALUE:
            self.__sendCommand('xvalue')
        elif value == CHART_TYPE_NONE:
            self.__sendCommand('noxvalue')
        else:
            raise ClichartError, 'Invalid chart type value: %s' % value

    def __sendCommand(self, command, arg = None, expectResponse = True):
        if DEBUG:
            print 'Command:', command, arg
        self.stdin.write(command)
        if arg:
            self.stdin.write(' %s' % arg)
        self.stdin.write('\n')
        self.stdin.flush()
        if not expectResponse:
            return
        self.__readResponse()

    def __readResponse(self):
        try:
            # Allow up to responseTimeout seconds to get the response (chart generation can take a while)
            try:
                response = self.queue.get(True, self.responseTimeout)
            except TypeError:
                # Python 2.2 doesn't support timeout
                response = self.queue.get(True)
            if DEBUG:
                print ' Response:', response
            if not response.startswith('OK'):
               raise ClichartError, response
        except Queue.Empty:
            raise ClichartError, 'No response received'
