#!/usr/bin/env python

"""
Unit tests for aggregate.py
"""

import unittest
import testUtils
from StringIO import StringIO

testUtils.checkPythonPath('aggregate')
from aggregate import *
import statslib

TEXT_INPUT = """10:23   58.5    47.3    2376    2020
10:24   57.8    47.2    2376    2017
10:25   58.1    47.4    2376    2018
10:26   58.4    47.3    2376    2020
"""

CSV_INPUT = """10:23, 58.5, 47.3, 2376, 2020
10:24, 57.8, 47.2, 2376, 2017
10:25, 58.1, 47.4, 2376, 2018
10:26, 58.4, 47.3, 2376, 2020
"""

CSV_INPUT_WITH_ZERO = '10:23, 0.12'

# ============================================================================
class ProcessFileTest(unittest.TestCase):
    def assertAlmostEqualList(self, expectedList, actualList):
        for expected, actual in zip(expectedList, actualList):
            self.assertAlmostEqual(expected, actual)
    def testProcessFile_EveryAggregateType_Text(self):
        output = StringIO()
        processFile(StringIO(TEXT_INPUT), output, ['1:min', '1:max', '1:av',
                '1:cnt', '1:tot', '1:first', '1:last'], False, False, False, [])
        results = [float(value) for value in output.getvalue().split()]
        self.assertAlmostEqualList([57.8, 58.5, 58.2, 4.0, 232.8, 58.5, 58.4], results)
    def testProcessFile_SimpleExpressions_Text(self):
        output = StringIO()
        processFile(StringIO(TEXT_INPUT), output, ['1:min - 57.8', '1:max / 1:max', '1:av / 10.0',
                '1:tot / 1:cnt'], False, False, False, [])
        results = [float(value) for value in output.getvalue().split()]
        self.assertAlmostEqualList([0.0, 1.0, 5.82, 58.2], results)
    def testProcessFile_ManyColumns_Csv(self):
        output = StringIO()
        processFile(StringIO(CSV_INPUT), output, ['1:min', '2:max', '3:av',
                '4:tot', '1:first', '2:last'], True, False, False, [])
        results = [float(value) for value in output.getvalue().split(',')]
        self.assertAlmostEqualList([57.8, 47.4, 2376, 8075, 58.5, 47.3], results)
    def testProcessFile_OneColumn_CsvWithHeader(self):
        output = StringIO()
        processFile(StringIO(CSV_INPUT), output, ['1:min'], True, True, False, [])
        results = [float(value) for value in output.getvalue().split(',')]
        self.assertAlmostEqualList([57.8], results)
    def testProcessFile_WithSinglePrefix(self):
        output = StringIO()
        processFile(StringIO(CSV_INPUT), output, ['1:min', '2:max'], True, True, False, ['12/03/2007'])
        resultList = output.getvalue().split(',')
        self.assertEqual('12/03/2007', resultList[0])
        results = [float(value) for value in resultList[1:]]
        self.assertAlmostEqualList([57.8, 47.4], results)
    def testProcessFile_EmptyAndSilent(self):
        output = StringIO()
        processFile(StringIO(''), output, ['1:min'], True, True, True, [])
        self.assertEqual('', output.getvalue())
    def testProcessFile_EmptyAndNotSilent(self):
        output = StringIO()
        try:
            processFile(StringIO(''), output, ['1:min'], True, True, False, [])
            fail()
        except statslib.InvalidDataException, expected:
            pass
    def testProcessFile_InvalidNumberInInput(self):
        output = StringIO()
        invalidInput = CSV_INPUT.replace('58.1', '58xxx1')
        try:
            processFile(StringIO(invalidInput), output, ['1:min'], True, False, False, [])
            self.fail()
        except statslib.InvalidDataException, expected:
            pass
    def testProcessFile_InvalidOutputColumnIndex(self):
        output = StringIO()
        try:
            processFile(StringIO(CSV_INPUT), output, ['5:min'], True, False, False, [])
            self.fail()
        except statslib.InvalidDataException, expected:
            pass
    def testProcessFile_InvalidOutputExpression(self):
        output = StringIO()
        try:
            processFile(StringIO(CSV_INPUT), output, ['2:min +/ 2'], True, False, False, [])
            self.fail()
        except statslib.InvalidOptionException, expected:
            pass
    def testProcessFile_WithZeroQuotient(self):
        output = StringIO()
        processFile(StringIO(CSV_INPUT_WITH_ZERO), output, ['1:min', '1:av', '1:tot'], True, False, False, [])
        resultList = output.getvalue().split(',')
        results = [float(value) for value in resultList]
        self.assertEqual(results, [0.12, 0.12, 0.12])




# ============================================================================
if __name__ == '__main__':
    unittest.main()
