#! /usr/bin/env python

"""
(C) Copyright 2006, by John Dickson

Project Info:  http://clichart.sourceforge.net/

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
USA.

$Id: statslib.py 517 2007-03-04 08:37:58Z johnd $


Library functions and classes for generating summary statistics from text data
"""

import sys, re, traceback


# ==============================================================================
class LineMatcher:
    """Given a line of text, return True if it should be included"""
    def __init__(self, pattern = None):
        self.regex = None
        if pattern:
            try:
                self.regex = re.compile(pattern)
            except:
                usage('Invalid regular expression [%s]' % pattern, True)

    def shouldInclude(self, line):
        if not self.regex:
            return True
        return self.regex.search(line) is not None

# ==============================================================================
class ValueExtractor:
    """Given a line of text, can extract and return a value from the line.
    Note that all indexes are 0-based"""
    def __init__(self, optionText):
        self.charStartIndex = None
        self.charEndIndex = None
        self.fieldIndex = None
        self.regex = None

        optionType, option = optionText.split(':', 1)
        if optionType =='s':
            try:
                indexes = option.split(':')
                self.charStartIndex = int(indexes[0])
                if len(indexes) > 1 and indexes[1]:
                    self.charEndIndex = int(indexes[1])
            except:
                #import traceback; traceback.print_exc()
                usage('Invalid substring specification [%s]' % option)
        elif optionType == 'f':
            try:
                self.fieldIndex = int(option)
            except:
                usage('Invalid field number [%s]' % option)
        elif optionType == 'r':
            try:
                self.regex = re.compile(option)
            except:
                usage('Invalid regular expression [%s]' % option, True)
        else:
            usage('Invalid field specification [%s]' % optionText)

    def getValue(self, line, lineNumber):
        if self.charEndIndex is not None:
            return line[self.charStartIndex:self.charEndIndex]
        if self.charStartIndex is not None:
            return line[self.charStartIndex:]
        elif self.fieldIndex is not None:
            return line.split()[self.fieldIndex]
        else:
            match = self.regex.search(line)
            if not match:
                print 'ERROR on line %d: Failed to match regular expression [%s]' % (lineNumber, self.regex.pattern)
                print 'Line text:', line
                sys.exit(1)
            if match.groups():
                return match.group(1)
            else:
                return match.group(0)

# ==============================================================================
class OutputFormatter:
    def __init__(self, isCsv, quote = False):
        self.isCsv = isCsv
        self.quote = quote

    def _quote(self, value):
        """Only quote if field contains comma or space"""
        if not self.quote:
            return value
        if ' ' not in value and ',' not in value:
            return value
        return '"%s"' % value.replace('"', '""')

    def _format(self, value):
        """Try to format numbers as right-justified over 8 chars"""
        try:
            # ask forgiveness
            float(value)
            return '%-8s' % value
        except:
            return '%8s' % self._quote(value)


    def output(self, key, values):
        if self.isCsv:
            if key is not None:
                print '%s,' % self._quote(key),
            print ', '.join([self._quote(str(value)) for value in values])
        else:
            if key is not None:
                print '%s ' % self._quote(key),
            print '  '.join([self._format(value) for value in values])

