/* (C) Copyright 2006, by John Dickson
 *
 * Project Info:  http://clichart.sourceforge.net/
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 * $Id: AbstractChartBuilder.java 539 2007-04-01 00:03:27Z johnd $
 */

package net.sf.clichart.chart;

import net.sf.clichart.data.DataSink;
import net.sf.clichart.data.InvalidDataException;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.XYToolTipGenerator;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.StackedXYBarRenderer;
import org.jfree.chart.renderer.xy.ClusteredXYBarRenderer;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYBarDataset;

import java.awt.*;

/**
 * Base class for objects that can build and return a chart based on a data series provided by the DataSink interface.
 *
 * Note that these objects can also be used to add a second axis to an existing chart, so when adding a second axis,
 * should create two of the same subclass (one for each axis), call getChart() on the first, and addSecondAxis() on
 * the second
 *
 * WARNING: Not threadsafe
 *
 * @author johnd
 */
public abstract class AbstractChartBuilder implements DataSink {

    /* ========================================================================
     *
     * Class (static) variables.
     */

    /* ========================================================================
     *
     * Instance variables.
     */

    private ColumnParseState m_columnParseState = new ColumnParseState();

    protected final Options m_options;

    /* ========================================================================
     *
     * Constructors
     */

    protected AbstractChartBuilder(Options options) {
        m_options = options;
    }

    /* ========================================================================
     *
     * Static methods
     */

    /* ========================================================================
     *
     * Public methods
     */

    // ------------------------------------------------------------------------
    // Methods from DataSink interface
    // ------------------------------------------------------------------------
    public void headerParsed(String[] headers) throws InvalidDataException {
        initialiseColumns(headers);
        m_columnParseState.setColumnsInitialised(true);
        m_columnParseState.setNumHeaders(headers.length);
    }

    public void dataParsed(Object xValue, Object[] yValues, int lineNumber) throws InvalidDataException {
        if (!m_columnParseState.isColumnsInitialised()) {
            initialiseColumns(new String[yValues.length]);
            m_columnParseState.setColumnsInitialised(true);
        }

        for (int i = 0; i < yValues.length; i++) {
            if (m_columnParseState.getNumHeaders() <= 0 || i < m_columnParseState.getNumHeaders()) {
                addYValue(i, xValue, yValues[i]);

            } else {
                // invalid column number
                if (!m_columnParseState.isExcessColumnsAdvised()) {
                    System.err.println("Invalid line data found on line " + lineNumber
                            + " - more columns than headers.  Ignoring excess columns.  "
                            + "No more warnings will be given");
                    m_columnParseState.setExcessColumnsAdvised(true);
                }
            }
        }
    }

    public void parsingFinished() {
        // don't really need to know?
    }

    // ------------------------------------------------------------------------
    // Method to create and return chart
    // ------------------------------------------------------------------------
    /**
     * Must be called *after* all parsing is finished
     * @return the chart ready to be embellished and displayed
     */
    public JFreeChart getChart(Options options) {
        assert m_columnParseState.isColumnsInitialised() : "Cannot get chart - no data provided yet";
        JFreeChart chart = getChartImpl(options);
        setOptions(chart, options);

        return chart;
    }

    /**
     * Add the data set as the second axis to this chart
     */
    public void addSecondAxis(JFreeChart chart, Options options) {
        XYPlot plot = chart.getXYPlot();
        plot.setDataset(1, getDataset());
        plot.mapDatasetToRangeAxis(1, 1);

        NumberAxis rangeAxis2 = new NumberAxis(options.getChartSecondYAxisTitle());
        plot.setRangeAxis(1, rangeAxis2);

        // need a separate renderer for the axis, otherwise the same colours are used as for the first axis series
        setAxisRenderer(chart.getXYPlot(), 1, options.isSecondAxisBarChart(), options.hasSecondAxisDataPoints(),
                options.getSecondAxisLineWeight());

        if (options.getSecondAxisMaxYValue() != -1 && rangeAxis2.getUpperBound() > options.getSecondAxisMaxYValue()) {
            rangeAxis2.setUpperBound(options.getSecondAxisMaxYValue());
        }
        rangeAxis2.setAutoRangeIncludesZero(false);
    }


    // ------------------------------------------------------------------------
    // Setters and getters
    // ------------------------------------------------------------------------

    /* ========================================================================
     *
     * Protected / package-private methods
     */

    /**
     * Called for subclass to create and register the appropriate data series for the column with this title
     * @param title
     */
    protected abstract void initialiseSeriesColumn(String title);

    /**
     * Called for subclass to add this yValue to the appropriate series
     * @param columnNumber The 0-based index into the dataColumnIndexes array
     * @param xValue The x axis value for this row
     * @param yValue The y axis value for this row and column number
     */
    protected abstract void addYValue(int columnNumber, Object xValue, Object yValue);

    /**
     * Called for subclass to create the required chart object, containing all the data provided
     *
     * @return the chart ready to be embellished and displayed
     */
    protected abstract JFreeChart getChartImpl(Options options);

    /**
     * Return the dataset containing all the data provided - used to add the second axis
     */
    protected abstract XYDataset getDataset();

    /**
     * Return an appropriate tooltip generator for a new renderer
     */
    protected abstract XYToolTipGenerator getToolTipGenerator();


    /* ========================================================================
     *
     * Private methods
     */

    private void initialiseColumns(String[] headers) {
        for (int i = 0; i < headers.length; i++) {
            String header = (headers[i] != null ? headers[i] : "Series " + (i + 1));
            initialiseSeriesColumn(header);
        }
    }

    private void setOptions(JFreeChart chart, Options options) {
        if (options.getMaxYValue() != -1) {
            ValueAxis rangeAxis = chart.getXYPlot().getRangeAxis();
            // Note that we only do this if the max y value has been exceeded
            if (rangeAxis.getUpperBound() > options.getMaxYValue()) {
                rangeAxis.setUpperBound(options.getMaxYValue());
            }
        }

        setAxisRenderer(chart.getXYPlot(), 0, options.isBarChart(), options.hasDataPoints(), options.getLineWeight());
    }

    private void setAxisRenderer(XYPlot plot, int axisIndex, boolean isBarChart, boolean hasDataPoints,
            int lineWeight) {
        XYItemRenderer renderer;
        if (isBarChart) {
            renderer = new ClusteredXYBarRenderer();
            XYDataset axisDataset = plot.getDataset(axisIndex);
            plot.setDataset(axisIndex, new XYBarDataset(axisDataset, calculateBarWidth(axisDataset, lineWeight)));

        } else {
            renderer = new XYLineAndShapeRenderer(true, hasDataPoints);

            if (lineWeight >= 0 && lineWeight <= 5) {
                for (int seriesIndex = 0; seriesIndex < plot.getDataset(axisIndex).getSeriesCount(); seriesIndex++) {
                    renderer.setSeriesStroke(seriesIndex, new BasicStroke((float)lineWeight));
                }
            }
        }
        renderer.setBaseToolTipGenerator(getToolTipGenerator());
        plot.setRenderer(axisIndex, renderer);
    }

    private double calculateBarWidth(XYDataset axisDataset, int lineWeight) {
        double minimumInterval = -1;
        for (int seriesIndex = 0; seriesIndex < axisDataset.getSeriesCount(); seriesIndex++) {
            for (int itemIndex = 1; itemIndex < axisDataset.getItemCount(seriesIndex); itemIndex++) {
                double interval = axisDataset.getXValue(seriesIndex, itemIndex)
                        - axisDataset.getXValue(seriesIndex, itemIndex - 1);
                if (itemIndex == 1) {
                    minimumInterval = interval;
                } else {
                    minimumInterval = Math.min(minimumInterval, interval);
                }
            }
        }

        double barWeight = 0.5;
        if (lineWeight >= 0 && lineWeight <= 5) {
            barWeight = 0.2 * lineWeight;
        }

        return minimumInterval * barWeight;
    }

}