/* (C) Copyright 2006, by John Dickson
 *
 * Project Info:  http://clichart.sourceforge.net/
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 *
 * $Id: OptionParser.java 545 2007-04-06 07:17:23Z johnd $
 */

package net.sf.clichart.main;

import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import net.sf.clichart.chart.Options;

import java.util.Iterator;

/**
 * Used to parse the command line into options and arguments.
 *
 * <p> Note: This class refers to the options parsed from the command-line as Options, while the Jakarta commons CLI
 * API uses the word to mean option definitions.  The commons usage is represented with a fully-qualified class name
 * in this class.
 *
 * @author johnd
 */
public class OptionParser {

    /* ========================================================================
     *
     * Class (static) variables.
     */

    // Each option element is {shortName, longName, description}
    private static final String[][] OPTIONS_WITH_ARGS = {
        {"d", "dateformat", "Format of date/time, in SimpleDateFormat notation. Defaults to 'HH:mm'"},
        {"l", "columnlist", "List of columns, comma-separated, 0-based.  X axis value (if any) must be first.  " +
            "Defaults to '0,1'"},
        {"o", "outputpath", "Output chart to the given path (otherwise shows in a window), must be JPG or PNG"},
        {"t", "title", "Title for the chart"},
        {"x", "xtitle", "Title for the X axis"},
        {"y", "ytitle", "Title for the Y axis"},
        {"m", "maxy", "Maximum value for y axis"},
        {"w", "width", "Chart width in pixels (defaults to 800)"},
        {"g", "height", "Chart height in pixels (defaults to 600)"},
        {null, "columnlist2", "List of columns for second y axis (if any), comma-separated, 0-based."},
        {null, "ytitle2", "Title for the second Y axis (if any)"},
        {null, "maxy2", "Maximum value for second y axis (if any)"},
        {null, "lineweight", "Line weight (values are 1 - 5)"},
        {null, "lineweight2", "Line weight for the second Y axis (values are 1 - 5)"},
    };

    private static final String[][] OPTIONS_WITHOUT_ARGS = {
        {"b", "bar", "Show as a bar chart, not X-Y line"},
        {"c", "csv", "Expect input as CSV.  Default is whitespace-separated"},
        {"f", "hasheader", "First row provides column headers for the legend (default is no header row)"},
        {"h", "help", "Show usage (this screen) and exit"},
        {"i", "ignoremissing", "Ignore missing columns (default is to terminate)"},
        {"p", "ignoredup", "Ignore duplicate X axis values (default is to terminate)"},
        {"n", "noxvalue", "Chart has no X axis values - just number the rows instead"},
        {"v", "xvalue", "Chart has simple values as the X axis, not dates or times"},
        {null, "bar2", "Show second axis as a bar chart, not X-Y line"},
        {null, "datapoints", "Indicate each data point"},
        {null, "datapoints2", "Indicate each data point for the second Y axis"},
    };



    /* ========================================================================
     *
     * Instance variables.
     */

    private org.apache.commons.cli.Options m_optionDefs;

    /* ========================================================================
     *
     * Constructors
     */

    public OptionParser() {
        m_optionDefs = buildOptionDefs();
    }

    /* ========================================================================
     *
     * Static methods
     */

    /* ========================================================================
     *
     * Public methods
     */

    public Options getOptions(String[] args) throws InvalidOptionsException, ShowUsageException {
        try {
            CommandLineParser clParser = new PosixParser();
            CommandLine commandLine = clParser.parse(m_optionDefs, args);
            return new CliOptions(commandLine);

        } catch (ParseException e) {
            throw new InvalidOptionsException("Invalid command line: " + e.getMessage(), e);
        }

    }

    public void showHelp(String message) {
        HelpFormatter formatter = new FixedHelpFormatter();

        String header = "Create a chart from tabular data";
        if (message != null) {
            header = message + "\n\n" + header;
        }

        formatter.printHelp("java -jar clichart.jar [options] [inputFile]", header, m_optionDefs,
                "If input file is omitted, reads data from stdin");

        exit();
    }

    /* ========================================================================
     *
     * Protected / package-private methods
     */

    /* ========================================================================
     *
     * Private methods
     */

    private org.apache.commons.cli.Options buildOptionDefs() {
        org.apache.commons.cli.Options optionDefs = new org.apache.commons.cli.Options();

        for (int i = 0; i < OPTIONS_WITH_ARGS.length; i++) {
            addOption(optionDefs, OPTIONS_WITH_ARGS[i], true);
        }

        for (int i = 0; i < OPTIONS_WITHOUT_ARGS.length; i++) {
            addOption(optionDefs, OPTIONS_WITHOUT_ARGS[i], false);
        }

        return optionDefs;
    }

    private void addOption(org.apache.commons.cli.Options optionDefs, String[] optionDetails, boolean hasArgument) {
        // OptionBuilder is a stateful class
        OptionBuilder.withLongOpt(optionDetails[1]);
        OptionBuilder.withDescription(optionDetails[2]);

        if (hasArgument) {
            OptionBuilder.hasArg(true);
            OptionBuilder.withValueSeparator();
        }

        if (optionDetails[0] != null) {
            optionDefs.addOption(OptionBuilder.create(optionDetails[0]));
        } else {
            optionDefs.addOption(OptionBuilder.create());
        }
    }

    // package-private, so can be overridden in test
    void exit() {
        System.exit(1);
    }


}